# coding: utf-8
"""
    dooku
    ~~~~~

    Dooku is a set of libraries for the Python programming language that
    provides various useful stuff that you might need in everyday usage.

    The idea is to provide a set of libraries and useful functions, just
    like Boost does in C++ world.

    Dooku is a good not only as a set of libraries, it also may be used
    as a some kind of incubator where various developers can test and
    evolve their projects. So please don't hesitate to propose your
    library to be a part of Dooku.

    :copyright: (c) 2014, Igor Kalnitsky
    :license: BSD, see LICENSE for details
"""

__version__ = '0.1.0'
__license__ = 'BSD'
